@@ -10,12 +10,17 @@ balances, and postings in each account.
 module Hledger.Data.Ledger
 where
 import Data.Map (Map, findWithDefault, fromList)
-import Hledger.Data.Utils
+import Data.Tree
+import Test.HUnit
+import Text.Printf
+
+import Hledger.Utils
 import Hledger.Data.Types
 import Hledger.Data.Account (nullacct)
 import Hledger.Data.AccountName
 import Hledger.Data.Journal
 import Hledger.Data.Posting
+import Hledger.Data.Matching
 
 
 instance Show Ledger where
@@ -41,6 +46,15 @@ journalToLedger fs j = nullledger{journal=j',accountnametree=t,accountmap=m}
     where j' = filterJournalPostings fs{depth=Nothing} j
           (t, m) = journalAccountInfo j'
 
+-- | Filter a journal's transactions as specified, and then process them
+-- to derive a ledger containing all balances, the chart of accounts,
+-- canonicalised commodities etc.
+-- Like journalToLedger but uses the new matchers.
+journalToLedger2 :: Matcher -> Journal -> Ledger
+journalToLedger2 m j = nullledger{journal=j',accountnametree=t,accountmap=amap}
+    where j' = filterJournalPostings2 m j
+          (t, amap) = journalAccountInfo j'
+
 -- | List a ledger's account names.
 ledgerAccountNames :: Ledger -> [AccountName]
 ledgerAccountNames = drop 1 . flatten . accountnametree
@@ -57,6 +71,10 @@ ledgerAccounts = drop 1 . flatten . ledgerAccountTree 9999
 ledgerTopAccounts :: Ledger -> [Account]
 ledgerTopAccounts = map root . branches . ledgerAccountTree 9999
 
+-- | List a ledger's bottom-level (subaccount-less) accounts, in tree order
+ledgerLeafAccounts :: Ledger -> [Account]
+ledgerLeafAccounts = leaves . ledgerAccountTree 9999
+
 -- | Accounts in ledger whose name matches the pattern, in tree order.
 ledgerAccountsMatching :: [String] -> Ledger -> [Account]
 ledgerAccountsMatching pats = filter (matchpats pats . aname) . accounts
